#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_workflow.interfaces import IWorkflowManagedContent
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.interfaces import IContainer
from zope.contentprovider.interfaces import IContentProvider

# import packages
from pyams_security.schema import PermissionField
from pyams_utils.schema import PersistentDict, PersistentList
from zope.container.constraints import contains
from zope.interface import invariant, Interface, Attribute, Invalid
from zope.schema import List, TextLine, Object, Int, Bool, Choice

from pyams_portal import _


class IPortlet(Interface):
    """Portlet interface"""

    name = Attribute("Portlet internal name")

    label = Attribute("Portlet visible name")

    permission = PermissionField(title="Portlet permission",
                                 description="Permission required to display permission",
                                 required=False)

    toolbar_image = Attribute("Porlet toolbar image")

    toolbar_css_class = Attribute("Portlet toolbar CSS class")


class IPortletAddingInfo(Interface):
    """Portlet adding info interface"""

    portlet_name = Choice(title=_("Portlet"),
                          vocabulary="PyAMS portal portlets")

    slot_name = Choice(title=_("Slot name"),
                       description=_("Slot name to which this configuration applies"),
                       required=True,
                       vocabulary='PyAMS template slots')


class IPortletConfiguration(Interface):
    """Portlet configuration interface"""

    template = Attribute("Template to which this configuration applis")

    slot_name = TextLine(title=_("Slot name"),
                         description=_("Slot name to which this configuration applies"),
                         required=True)

    portlet_name = Attribute("Portlet name")

    position = Int(title=_("Position"),
                   description=_("Portlet position inside slot"),
                   required=True,
                   min=0)

    visible = Bool(title=_("Visible portlet?"),
                   description=_("Select 'no' to hide this portlet. This will not break configuration inheritance..."),
                   required=True,
                   default=True)

    can_inherit = Attribute("Can inherit parent configuration?")

    inherit_parent = Bool(title=_("Inherit parent configuration?"),
                          description=_("This option is only available if context's parent is using the same template "
                                        "and if this portlet is also present in the same slot..."),
                          required=True,
                          default=True)


class IPortletContentProvider(IContentProvider):
    """Portlet content provider"""

    portlet = Object(title="Portlet utility",
                     schema=IPortlet)

    configuration = Object(title="Portlet renderer configuration",
                           schema=IPortletConfiguration)


class IPortletPreviewer(IPortletContentProvider):
    """Portlet previewer interface

    A portlet previewer should be defined as an adapter for a context,
    a request, a view and a portlet
    """


class IPortletRenderer(IPortletContentProvider):
    """Portlet renderer interface

    A portlet renderer should be defined as an adapter for a context,
    a request, a view and a portlet
    """


class ISlot(Interface):
    """Portal template slot interface"""

    name = TextLine(title=_("Slot name"),
                    description=_("This name must be unique in a given template"),
                    required=True)

    row_id = Int(title=_("Row ID"),
                 required=False)


class ISlotConfiguration(Interface):
    """Portal slot configuration"""

    template = Attribute("Slot template")

    slot_name = TextLine(title="Slot name")

    visible = Bool(title=_("Visible slot?"),
                   description=_("Select 'no' to hide this slot. This will not break configuration inheritance..."),
                   required=True,
                   default=True)

    can_inherit = Attribute("Can inherit parent configuration?")

    inherit_parent = Bool(title=_("Inherit parent configuration?"),
                          description=_("This option is only available if context's parent template is using a "
                                        "template containing the same slot..."),
                          required=True,
                          default=True)

    xs_width = Int(title=_("Extra small device width"),
                   description=_("Slot width, in columns count, on extra small devices (phones...); "
                                 "set to 0 to hide the portlet"),
                   required=False,
                   min=0,
                   max=12)

    sm_width = Int(title=_("Small device width"),
                   description=_("Slot width, in columns count, on small devices (tablets...); "
                                 "set to 0 to hide the portlet"),
                   required=False,
                   min=0,
                   max=12)

    md_width = Int(title=_("Medium devices width"),
                   description=_("Slot width, in columns count, on medium desktop devices (>= 992 pixels); "
                                 "set to 0 to hide the portlet"),
                   required=False,
                   min=0,
                   max=12)

    lg_width = Int(title=_("Large devices width"),
                   description=_("Slot width, in columns count, on large desktop devices (>= 1200 pixels); "
                                 "set to 0 to hide the portlet"),
                   required=False,
                   min=0,
                   max=12)

    css_class = TextLine(title=_("CSS class"),
                         description=_("CSS class applied to this slot"),
                         required=False)

    def get_css_class(self, device=None):
        """Get current CSS class"""

    def get_width(self, device=None):
        """Get slot width for each or given device"""

    def set_width(self, width, device=None):
        """Set width in columns count for given device"""


class ISlotRenderer(IContentProvider):
    """Slot renderer"""


class IPortalTemplateConfiguration(Interface):
    """Portal template configuration interface"""

    rows = Int(title="Rows count",
               required=True,
               default=1,
               min=0)

    def add_row(self):
        """Add new row"""

    def set_row_order(self, order):
        """Change template rows order"""

    def delete_row(self, row_id):
        """Delete template row"""

    slot_names = PersistentList(title="Slot names",
                                value_type=TextLine())

    slot_order = PersistentDict(title="Solts order",
                                key_type=Int(),  # row index
                                value_type=PersistentList(value_type=TextLine()))  # slot name

    slots = PersistentDict(title="Slots portlets",
                           description="List of slots associated with a given template",
                           key_type=Int(),  # row index
                           value_type=PersistentDict(key_type=TextLine(),  # slot name
                                                     value_type=PersistentList(value_type=Choice(
                                                         vocabulary='PyAMS portal portlets')),  # portlet names
                                                     required=False))

    slot_config = PersistentDict(title="Slots configuration",
                                 key_type=TextLine(),  # slot name
                                 value_type=Object(schema=ISlotConfiguration),
                                 required=False)

    def add_slot(self, slot_name, row_id=None):
        """Add slot with given name"""

    def set_slot_order(self, order):
        """Change template slots order"""

    def get_slot_row(self, slot_name):
        """Get row containing given slot"""

    def get_slots(self, row_id):
        """Get ordered list of slots for given row ID"""

    def get_slots_width(self, device=None):
        """Get slots width for given or all device(s)"""

    def set_slot_width(self, slot_name, device, width):
        """Set slot width for given device"""

    def get_slot_configuration(self, slot_name):
        """Get slot configuration for given slot"""

    def delete_slot(self, slot_name):
        """Delete template slot"""


# class IPortalPortletsConfiguration(Interface):
#     """Portal template portlets configuration interface"""
#
    portlet_config = PersistentDict(title="Portlet configuration",
                                    key_type=TextLine(),  # slot name
                                    value_type=PersistentDict(key_type=Int(min=0),  # portlet position inside slot
                                                              value_type=Object(schema=IPortletConfiguration)),
                                    required=False)

    def add_portlet(self, portlet_name, slot_name):
        """Add portlet to givben slot"""

    def set_portlet_order(self, order):
        """Set template portlets order"""

    def get_portlet_configuration(self, slot_name, position):
        """Get portlet configuration for given slot"""

    def delete_portlet(self, slot_name, position):
        """Delete template portlet"""


class IPortalTemplate(IAttributeAnnotatable):
    """Portal template interface

    A portal template is a named utility providing a name and a set of slots.
    """

    name = TextLine(title=_("Template name"),
                    description=_("Two registered templates can't share the same name..."),
                    required=True)


class IPortalWfTemplate(IWorkflowManagedContent):
    """Workflow managed portal template interface"""


class IPortalTemplateContainer(IContainer, IAttributeAnnotatable):
    """Portal template container interface"""

    contains(IPortalTemplate)


class IPortalTemplateContainerConfiguration(Interface):
    """Portal templates container configuration"""

    selected_portlets = List(title=_("Selected portlets"),
                             description=_("These portlets will be directly available in templates configuration "
                                           "page toolbar"),
                             value_type=Choice(vocabulary="PyAMS portal portlets"),
                             required=False)


class IPortalTemplateRenderer(IContentProvider):
    """Portal template renderer

    A portal template renderer should be implemented as an adapter for a context, a request
    and a template
    """


class IPortalPage(Interface):
    """Portal page interface

    The page is the highest configuration level.
    It defines which template is used (a shared or local one), which gives
    the slots list.
    """

    can_inherit = Attribute("Can inherit parent template?")

    inherit_parent = Bool(title=_("Inherit parent template?"),
                          description=_("Should we reuse parent template?"),
                          required=True,
                          default=True)

    use_local_template = Bool(title=_("Use local template?"),
                              description=_("If 'yes', you can define a custom local template instead of "
                                            "a shared template"),
                              required=True,
                              default=False)

    shared_template = Choice(title=_("Page template"),
                             description=_("Template used for this page"),
                             vocabulary='PyAMS portal templates',
                             required=False)

    @invariant
    def check_template(self):
        if not (self.use_local_template or self.shared_template):
            raise Invalid(_("You must choose to use a local template or select a shared one!"))

    local_template = Object(title=_("Local template"),
                            schema=IPortalWfTemplate,
                            required=False)

    template = Attribute("Used template")


class IPortalContext(IAttributeAnnotatable):
    """Portal context marker interface"""
